@@ -28,8 +28,8 @@
 #define LIBPOKE_API
 #endif
 
-typedef struct pk_compiler *pk_compiler;
-typedef struct pk_ios *pk_ios;
+typedef struct _pk_compiler *pk_compiler;
+typedef struct _pk_ios *pk_ios;
 typedef uint64_t pk_val;
 
 /* The following status codes are returned by pk_errno function.  */
@@ -65,12 +65,12 @@ struct pk_term_if
   void (*indent_fn) (unsigned int lvl, unsigned int step);
 
   /* Mark the beginning of a styling class with name CLASS.  */
-  void (*class_fn) (const char *class);
+  void (*class_fn) (const char *aclass);
 
   /* Mark the end of a styling class with name CLASS.  This function
      returns 0 if the given class is not active and therefore can't be
      ended.  1 otherwise.  */
-  int (*end_class_fn) (const char *class);
+  int (*end_class_fn) (const char *aclass);
 
   /* Mark the beginning of an hyperlink with url URL and identifier
      ID.  The identifier can be NULL.  */
@@ -161,7 +161,9 @@ int pk_compile_statement (pk_compiler pkc, const char *buffer,
    expression, which evaluates to a value.
 
    VAL, if given, is a pointer to a pk_val variable that is set to the
-   result value of executing the expression.  */
+   result value of executing the expression.
+
+   Return PK_ERROR in case of a compilation error, PK_OK otherwise.  */
 
 int pk_compile_expression (pk_compiler pkc, const char *buffer,
                            const char **end, pk_val *val) LIBPOKE_API;
@@ -409,7 +411,7 @@ int pk_defvar (pk_compiler pkc, const char *varname, pk_val val) LIBPOKE_API;
 
    CLS is the closure for the function to call.
 
-   RET is set to the value returned by the function, or to PK_NULL if
+   RET is set to the value returned by the function, or to NULL if
    it is a void function.
 
    A variable number of function arguments follow, terminated by
