@@ -27,7 +27,7 @@
 #include "xalloc.h"
 
 #include "pvm.h"
-#include "pvm-alloc.h" /* For pvm_alloc_{add/remove}_gc_roots */
+#include "pvm-alloc.h" /* For pvm_{alloc,free}_uncollectable.  */
 #include "pk-utils.h"
 #include "pkl-ast.h"
 
@@ -370,6 +370,7 @@ pkl_ast_node
 pkl_ast_make_array_type (pkl_ast ast, pkl_ast_node etype, pkl_ast_node bound)
 {
   pkl_ast_node type = pkl_ast_make_type (ast);
+  const int nclosures = 5; /* mapper, writer, bounder, constructor, printer */
 
   assert (etype);
 
@@ -378,19 +379,15 @@ pkl_ast_make_array_type (pkl_ast ast, pkl_ast_node etype, pkl_ast_node bound)
   if (bound)
     PKL_AST_TYPE_A_BOUND (type) = ASTREF (bound);
 
+  /* Prevent GC to collect these PVM values.  */
+  PKL_AST_TYPE_A_CLOSURES (type)
+      = pvm_alloc_uncollectable (nclosures * sizeof (pvm_val));
   PKL_AST_TYPE_A_MAPPER (type) = PVM_NULL;
   PKL_AST_TYPE_A_WRITER (type) = PVM_NULL;
   PKL_AST_TYPE_A_BOUNDER (type) = PVM_NULL;
   PKL_AST_TYPE_A_CONSTRUCTOR (type) = PVM_NULL;
   PKL_AST_TYPE_A_PRINTER (type) = PVM_NULL;
 
-  /* The closure slots are GC roots.  */
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_A_MAPPER (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_A_WRITER (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_A_BOUNDER (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_A_CONSTRUCTOR (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_A_PRINTER (type), 1);
-
   return type;
 }
 
@@ -444,6 +441,8 @@ pkl_ast_make_struct_type (pkl_ast ast,
                           int pinned_p, int union_p)
 {
   pkl_ast_node type = pkl_ast_make_type (ast);
+  const int nclosures
+      = 6; /* writer, mapper, constructor, comparator, integrator, printer.  */
 
   PKL_AST_TYPE_CODE (type) = PKL_TYPE_STRUCT;
   PKL_AST_TYPE_S_NELEM (type) = nelem;
@@ -455,6 +454,10 @@ pkl_ast_make_struct_type (pkl_ast ast,
     PKL_AST_TYPE_S_ITYPE (type) = ASTREF (itype);
   PKL_AST_TYPE_S_PINNED_P (type) = pinned_p;
   PKL_AST_TYPE_S_UNION_P (type) = union_p;
+
+  /* Prevent GC to collect these PVM values.  */
+  PKL_AST_TYPE_S_CLOSURES (type)
+      = pvm_alloc_uncollectable (nclosures * sizeof (pvm_val));
   PKL_AST_TYPE_S_MAPPER (type) = PVM_NULL;
   PKL_AST_TYPE_S_WRITER (type) = PVM_NULL;
   PKL_AST_TYPE_S_CONSTRUCTOR (type) = PVM_NULL;
@@ -462,14 +465,6 @@ pkl_ast_make_struct_type (pkl_ast ast,
   PKL_AST_TYPE_S_INTEGRATOR (type) = PVM_NULL;
   PKL_AST_TYPE_S_PRINTER (type) = PVM_NULL;
 
-  /* The closure slots are GC roots.  */
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_WRITER (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_MAPPER (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_CONSTRUCTOR (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_COMPARATOR (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_INTEGRATOR (type), 1);
-  pvm_alloc_add_gc_roots (&PKL_AST_TYPE_S_PRINTER (type), 1);
-
   return type;
 }
 
@@ -1244,34 +1239,31 @@ pkl_type_append_to (pkl_ast_node type, int use_given_name,
       {
         pkl_ast_node t;
 
-        if (PKL_AST_TYPE_F_NARG (type) > 0)
+        sb_append (buffer, "(");
+        for (t = PKL_AST_TYPE_F_ARGS (type); t;
+             t = PKL_AST_CHAIN (t))
           {
-            sb_append (buffer, "(");
+            pkl_ast_node atype
+              = PKL_AST_FUNC_TYPE_ARG_TYPE (t);
 
-            for (t = PKL_AST_TYPE_F_ARGS (type); t;
-                 t = PKL_AST_CHAIN (t))
+            if (t != PKL_AST_TYPE_F_ARGS (type))
+              sb_append (buffer, ",");
+
+            if (PKL_AST_FUNC_TYPE_ARG_VARARG (t))
+              sb_append (buffer, "...");
+            else
               {
-                pkl_ast_node atype
-                  = PKL_AST_FUNC_TYPE_ARG_TYPE (t);
-
-                if (PKL_AST_FUNC_TYPE_ARG_VARARG (t))
-                  sb_append (buffer, "...");
-                else
-                  {
-                    if (t != PKL_AST_TYPE_F_ARGS (type))
-                      sb_append (buffer, ",");
-                    pkl_type_append_to (atype, use_given_name, buffer);
-                    if (PKL_AST_FUNC_TYPE_ARG_OPTIONAL (t))
-                      sb_append (buffer, "?");
-                  }
+                if (t != PKL_AST_TYPE_F_ARGS (type))
+                  sb_append (buffer, ",");
+                pkl_type_append_to (atype, use_given_name, buffer);
+                if (PKL_AST_FUNC_TYPE_ARG_OPTIONAL (t))
+                  sb_append (buffer, "?");
               }
-
-            sb_append (buffer, ")");
           }
+        sb_append (buffer, ")");
 
         pkl_type_append_to (PKL_AST_TYPE_F_RTYPE (type), use_given_name,
                             buffer);
-        sb_append (buffer, ":");
         break;
       }
     case PKL_TYPE_OFFSET:
@@ -1987,24 +1979,13 @@ pkl_ast_node_free (pkl_ast_node ast)
       switch (PKL_AST_TYPE_CODE (ast))
         {
         case PKL_TYPE_ARRAY:
-          /* Remove GC roots.  */
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_A_MAPPER (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_A_WRITER (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_A_BOUNDER (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_A_CONSTRUCTOR (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_A_PRINTER (ast), 1);
+          pvm_free_uncollectable (PKL_AST_TYPE_A_CLOSURES (ast));
 
           pkl_ast_node_free (PKL_AST_TYPE_A_BOUND (ast));
           pkl_ast_node_free (PKL_AST_TYPE_A_ETYPE (ast));
           break;
         case PKL_TYPE_STRUCT:
-          /* Remove GC roots.  */
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_WRITER (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_MAPPER (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_CONSTRUCTOR (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_COMPARATOR (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_INTEGRATOR (ast), 1);
-          pvm_alloc_remove_gc_roots (&PKL_AST_TYPE_S_PRINTER (ast), 1);
+          pvm_free_uncollectable (PKL_AST_TYPE_S_CLOSURES (ast));
 
           for (t = PKL_AST_TYPE_S_ELEMS (ast); t; t = n)
             {
