@@ -40,27 +40,47 @@ static pvm_val string_type;
 static pvm_val void_type;
 static pvm_val any_type;
 
+/* We are currently only supporting a relatively small number of
+   integral types, i.e. signed and unsigned types of sizes 1 to 64
+   bits.  It is therefore possible to cache these types to avoid
+   allocating them again and again.
+
+   The contents of this table are initialized in pvm_val_initialize
+   and finalized in pvm_val_finalize.  New types are installed by
+   pvm_make_integral_type.
+
+   Note that the first entry of the table is unused; it would
+   correspond to an integer of "zero" bits.  This is more efficient
+   than correcting the index at every access at the cost of only
+   64-bits.  */
+
+static pvm_val common_int_types[65][2];
+
 pvm_val
 pvm_make_int (int32_t value, int size)
 {
+  assert (0 < size && size <= 32);
   return PVM_MAKE_INT (value, size);
 }
 
 pvm_val
 pvm_make_uint (uint32_t value, int size)
 {
+  assert (0 < size && size <= 32);
   return PVM_MAKE_UINT (value, size);
 }
 
 pvm_val
 pvm_make_long (int64_t value, int size)
 {
+  assert (0 < size && size <= 64);
   return PVM_MAKE_LONG_ULONG (value, size, PVM_VAL_TAG_LONG);
 }
 
 pvm_val
 pvm_make_ulong (uint64_t value, int size)
 {
+  assert (0 < size && size <= 64);
   return PVM_MAKE_LONG_ULONG (value, size, PVM_VAL_TAG_ULONG);
 }
 
@@ -82,6 +102,15 @@ pvm_make_string (const char *str)
   return PVM_BOX (box);
 }
 
+pvm_val
+pvm_make_string_nodup (char *str)
+{
+  pvm_val_box box = pvm_make_box (PVM_VAL_TAG_STR);
+
+  PVM_VAL_BOX_STR (box) = str;
+  return PVM_BOX (box);
+}
+
 pvm_val
 pvm_make_array (pvm_val nelem, pvm_val type)
 {
@@ -184,26 +213,28 @@ pvm_array_set (pvm_val arr, pvm_val idx, pvm_val val)
 {
   size_t index = PVM_VAL_ULONG (idx);
   size_t nelem = PVM_VAL_ULONG (PVM_VAL_ARR_NELEM (arr));
-  size_t elem_boffset;
   size_t i;
+  ssize_t size_diff;
 
   /* Make sure that the given index is within bounds.  */
   if (index >= nelem)
     return 0;
 
+  /* Calculate the difference of size introduced by the new
+     elemeent.  */
+  size_diff = ((ssize_t) pvm_sizeof (val)
+               - (ssize_t) pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, index)));
+
   /* Update the element with the given value.  */
   PVM_VAL_ARR_ELEM_VALUE (arr, index) = val;
 
   /* Recalculate the bit-offset of all the elemens following the
      element just updated.  */
-  elem_boffset
-    = (PVM_VAL_ULONG (PVM_VAL_ARR_ELEM_OFFSET (arr, index))
-       + pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, index)));
-
   for (i = index + 1; i < nelem; ++i)
     {
+      size_t elem_boffset
+        = (ssize_t) PVM_VAL_ULONG (PVM_VAL_ARR_ELEM_OFFSET (arr, i)) + size_diff;
       PVM_VAL_ARR_ELEM_OFFSET (arr, i) = pvm_make_ulong (elem_boffset, 64);
-      elem_boffset += pvm_sizeof (PVM_VAL_ARR_ELEM_VALUE (arr, i));
     }
 
   return 1;
@@ -401,8 +432,8 @@ pvm_make_type (enum pvm_type_code code)
   return PVM_BOX (box);
 }
 
-pvm_val
-pvm_make_integral_type (pvm_val size, pvm_val signed_p)
+static inline pvm_val
+pvm_make_integral_type_1 (pvm_val size, pvm_val signed_p)
 {
   pvm_val itype = pvm_make_type (PVM_TYPE_INTEGRAL);
 
@@ -411,6 +442,22 @@ pvm_make_integral_type (pvm_val size, pvm_val signed_p)
   return itype;
 }
 
+pvm_val
+pvm_make_integral_type (pvm_val size, pvm_val signed_p)
+{
+  uint64_t bits = PVM_VAL_ULONG (size);
+  int32_t sign = PVM_VAL_INT (signed_p);
+  pvm_val itype = common_int_types[bits][sign];
+
+  if (itype == PVM_NULL)
+    {
+      itype = pvm_make_integral_type_1 (size, signed_p);
+      common_int_types[bits][sign] = itype;
+    }
+
+  return itype;
+}
+
 pvm_val
 pvm_make_string_type (void)
 {
@@ -1654,7 +1701,8 @@ pvm_make_exception (int code, char *message, int exit_status)
   pvm_allocate_struct_attrs (nfields, &field_names, &field_types);
 
   field_names[0] = code_name;
-  field_types[0] = pvm_make_integral_type (32, 1);
+  field_types[0] = pvm_make_integral_type (pvm_make_ulong (32, 64),
+                                           PVM_MAKE_INT (1, 32));
 
   field_names[1] = msg_name;
   field_types[1] = pvm_make_string_type ();
@@ -1688,13 +1736,21 @@ pvm_val_cls_program (pvm_val cls)
 void
 pvm_val_initialize (void)
 {
+  uint64_t i;
+  int32_t j;
+
   pvm_alloc_add_gc_roots (&string_type, 1);
   pvm_alloc_add_gc_roots (&void_type, 1);
   pvm_alloc_add_gc_roots (&any_type, 1);
+  pvm_alloc_add_gc_roots (&common_int_types, 65 * 2);
 
   string_type = pvm_make_type (PVM_TYPE_STRING);
   void_type = pvm_make_type (PVM_TYPE_VOID);
   any_type = pvm_make_type (PVM_TYPE_ANY);
+
+  for (i = 0; i < 65; ++i)
+    for (j = 0; j < 2; ++j)
+      common_int_types[i][j] = PVM_NULL;
 }
 
 void
@@ -1703,4 +1759,5 @@ pvm_val_finalize (void)
   pvm_alloc_remove_gc_roots (&string_type, 1);
   pvm_alloc_remove_gc_roots (&void_type, 1);
   pvm_alloc_remove_gc_roots (&any_type, 1);
+  pvm_alloc_remove_gc_roots (&common_int_types, 65 * 2);
 }
